#!/usr/bin/perl -w

use strict;
use warnings;
use GD::Graph::mixed;  #GDGraph installed using ppm qw/linespoints mixed/

use Stat;

#-----------------------#
#     Initialization    #
#-----------------------#

my @data1;
my @data2;

my $label1 = 1;
my $label2 = 2;

my $min = 0;
my $max = 1;
my $gap = 0.01;

my $SP = $min;

#-------------------#
#     Read data     #
#-------------------#

open IN, "< data.txt" or die "Could not open data file. $!";

while (<IN>){
    push @data1, (split /\t/,$_ )[1] if $_ =~ /^$label1/;
    chomp $_ foreach @data1;
    push @data2, (split /\t/,$_ )[1] if $_ =~ /^$label2/;
    chomp $_ foreach @data2;
}

close IN;

#-----------------------------#
#     MW U test clculation    #
#-----------------------------#

my ($umw,$zmw,$pmw) = &Stat::mannwhitney(\@data1,\@data2);
$umw = sprintf("%.3f",$umw);
$zmw = sprintf("%.3f",$zmw);
$pmw = sprintf("%.5f",$pmw);

open OUT, "> data_out.csv" or die "Cannot open data_out.txt $!";
print "Mann Whitney U test\nU\t$umw\nz\t$zmw\np\t$pmw\n\n";
print OUT "Mann Whitney U test\nU,$umw\nz,$zmw\np,$pmw\n\n";

#--------------------------#
#     Sp, Se clculation    #
#--------------------------#

my (%hist1, %hist2);
my ($neg_cnt,$pos_cnt) =(0,0);
my $tot_neg = $#data1+1;
my $tot_pos = $#data2+1;

my (@SP_array,@se_array,@sp_array,@LR_array);

print "Cut-off\tSe\tSp\tLR\n";
print OUT "Cut-off,Se,Sp,LR\n";

while ($SP < $max){
    my $tmp_SP = $SP;
    $SP += $gap;
    
    # Turbulate data
    foreach my $datum1 (@data1){
        $hist1{$SP}++ if ($datum1 <= $SP && $datum1 > $tmp_SP);
        $neg_cnt++ if ($datum1 <= $SP && $datum1 > $tmp_SP);
    }
    foreach my $datum2 (@data2){
        $hist2{$SP}++ if ($datum2 <= $SP && $datum2 > $tmp_SP);
        $pos_cnt++ if ($datum2 <= $SP && $datum2 > $tmp_SP);
    }
    
    # Calculate Se, Sp, and LR
    my $se = ($tot_pos-$pos_cnt)/$tot_pos ;
    my $sp = $neg_cnt/$tot_neg;
    my $LR;
    unless ($sp==1){
        $LR = $se/(1-$sp);
    }else{$LR = 'max';}
    
    my $fSP = sprintf("%2.3f", $SP);
    my $fsp = sprintf("%2.3f", $sp);
    my $fse = sprintf("%2.3f", $se);
    my $fLR;
    if ($LR ne 'max'){
        $fLR = sprintf("%2.3f", $LR);
    }else{
        $fLR = $LR;
    }
    print "$fSP\t$fse\t$fsp\t$fLR\n";
    print OUT "$fSP,$fse,$fsp,$fLR\n";
    
    # For GD::Graph
    push @SP_array, $fSP;
    push @se_array, $fse;
    push @sp_array, $fsp;
    #push @LR_array, eval{$LR/35};
}

close OUT;

#-------------------#
#   Format graph    #
#-------------------#

# Prepare unnamed array for GD::Graph;
my @data = (\@SP_array, \@se_array, \@sp_array  ); #)

my $graph = GD::Graph::mixed->new( 400, 300 );

$graph->set(
    t_margin => '20',
    b_margin => '20',
    l_margin => '20',
    r_margin => '20',
    title => "Sp,Se vs Cut-off",
    x_label => 'Cut-Off',
    x_tick_number => 10,
    x_axis_skip => 'auto',
    x_max_value => $max,
    #two_axes => 1, do not work with more than 2 datasets
    #use_axes => [1,2,2],
    dclrs => ['blue', 'green'],
    y_label => 'Sp,Se',
    #y2_label => 'LR',
    marker_size => 2,
    types => [ qw( linespoints linespoints ) ], # for mixed
    y_max_value =>1.2,
    #y2_max_value =>40
);

GD::Text->font_path("./");
$graph->set_x_axis_font("ARIAL",6);

$graph->set_legend(
    'Se',
    'Sp'
);

#-------------------#
#   Print graph     #
#-------------------#

my $image = $graph->plot( \@data );

open( OUT, "> graph.jpg") or die( "Cannot open file: graph.jpg" );
binmode OUT;
print OUT $image->jpeg();
close OUT;

system("start graph.jpg");

__END__

    